@@ -76,47 +76,75 @@ _OgreBulletExport btCylinderShape* createCylinderCollider(const MovableObject* m
 struct _OgreBulletExport CollisionListener
 {
     virtual ~CollisionListener() {}
+    /** Called when two objects collide
+    * @param other the other object
+    * @param manifoldPoint the collision point
+    */
     virtual void contact(const MovableObject* other, const btManifoldPoint& manifoldPoint) = 0;
 };
 
 struct _OgreBulletExport RayResultCallback
 {
     virtual ~RayResultCallback() {}
+    /** Called for each object hit by the ray
+    * @param other the other object
+    * @param distance the distance from ray origin to hit point
+    */
     virtual void addSingleResult(const MovableObject* other, float distance) = 0;
 };
 
 /// simplified wrapper with automatic memory management
-class _OgreBulletExport DynamicsWorld
+class _OgreBulletExport CollisionWorld
 {
+protected:
     std::unique_ptr<btCollisionConfiguration> mCollisionConfig;
     std::unique_ptr<btCollisionDispatcher> mDispatcher;
-    std::unique_ptr<btConstraintSolver> mSolver;
     std::unique_ptr<btBroadphaseInterface> mBroadphase;
-    btDynamicsWorld* mBtWorld;
+
+    btCollisionWorld* mBtWorld;
+
+public:
+    CollisionWorld(btCollisionWorld* btWorld) : mBtWorld(btWorld) {}
+    virtual ~CollisionWorld();
+
+    btCollisionObject* addCollisionObject(Entity* ent, ColliderType ct, int group = 1, int mask = -1);
+
+    void rayTest(const Ray& ray, RayResultCallback* callback, float maxDist = 1000);
+};
+
+/// simplified wrapper with automatic memory management
+class _OgreBulletExport DynamicsWorld : public CollisionWorld
+{
+    std::unique_ptr<btConstraintSolver> mSolver;
 
 public:
     explicit DynamicsWorld(const Vector3& gravity);
-    ~DynamicsWorld();
-    DynamicsWorld(btDynamicsWorld* btWorld) : mBtWorld(btWorld) {}
+    DynamicsWorld(btDynamicsWorld* btWorld) : CollisionWorld(btWorld) {}
 
+    /** Add an Entity as a rigid body to the DynamicsWorld
+    * @param mass the mass of the object
+    * @param ent the entity to control
+    * @param ct the collider type
+    * @param listener a listener to call on collision with other objects
+    * @param group the collision group
+    * @param mask the collision mask
+    */
     btRigidBody* addRigidBody(float mass, Entity* ent, ColliderType ct, CollisionListener* listener = nullptr,
                               int group = 1, int mask = -1);
 
-    btDynamicsWorld* getBtWorld() const { return mBtWorld; }
-
-    void rayTest(const Ray& ray, RayResultCallback* callback, float maxDist = 1000);
+    btDynamicsWorld* getBtWorld() const { return (btDynamicsWorld*)mBtWorld; }
 };
 
 class _OgreBulletExport DebugDrawer : public btIDebugDraw
 {
     SceneNode* mNode;
-    btDynamicsWorld* mWorld;
+    btCollisionWorld* mWorld;
 
     ManualObject mLines;
     int mDebugMode;
 
 public:
-    DebugDrawer(SceneNode* node, btDynamicsWorld* world)
+    DebugDrawer(SceneNode* node, btCollisionWorld* world)
         : mNode(node), mWorld(world), mLines(""), mDebugMode(DBG_DrawWireframe)
     {
         mLines.setCastShadows(false);
