@@ -16,7 +16,7 @@ namespace RTShader
 const String SRS_COOK_TORRANCE_LIGHTING = "CookTorranceLighting";
 
 //-----------------------------------------------------------------------
-CookTorranceLighting::CookTorranceLighting() : mLightCount(0), mMRMapSamplerIndex(0) {}
+CookTorranceLighting::CookTorranceLighting() : mLightCount(0), mMRMapSamplerIndex(0), mLtcLUT1SamplerIndex(-1) {}
 
 //-----------------------------------------------------------------------
 const String& CookTorranceLighting::getType() const { return SRS_COOK_TORRANCE_LIGHTING; }
@@ -116,7 +116,7 @@ bool CookTorranceLighting::createCpuSubPrograms(ProgramSet* programSet)
 
     fstage.mul(In(diffuse).xyz(), In(outDiffuse).xyz(), baseColor);
     fstage.assign(Vector3(0), Out(outDiffuse).xyz());
-    fstage.assign(In(diffuse).w(), Out(outDiffuse).w()); // forward alpha
+    fstage.mul(In(diffuse).w(), In(outDiffuse).w(), Out(outDiffuse).w()); // forward alpha
 
     fstage.callFunction("PBR_MakeParams", {In(baseColor), In(mrparams), InOut(pixelParams)});
 
@@ -133,10 +133,17 @@ bool CookTorranceLighting::createCpuSubPrograms(ProgramSet* programSet)
                                        In(lightDiffuse),     In(pointParams), In(lightDirView),      In(spotParams),
                                        In(pixelParams),      InOut(outDiffuse).xyz()};
 
+        if(mLtcLUT1SamplerIndex > -1)
+        {
+            auto ltcLUT1 = psProgram->resolveParameter(GCT_SAMPLER2D, "ltcLUT1Sampler", mLtcLUT1SamplerIndex);
+            auto ltcLUT2 = psProgram->resolveParameter(GCT_SAMPLER2D, "ltcLUT2Sampler", mLtcLUT1SamplerIndex + 1);
+            params.insert(params.begin(), {In(ltcLUT1), In(ltcLUT2)});
+            psProgram->addPreprocessorDefines("HAVE_AREA_LIGHTS");
+        }
+
         if (auto shadowFactor = psMain->getLocalParameter("lShadowFactor"))
         {
             params.insert(params.begin(), In(shadowFactor));
-            psProgram->addPreprocessorDefines("HAVE_SHADOW_FACTOR");
         }
 
         fstage.callFunction("PBR_Lights", params);
@@ -161,6 +168,9 @@ bool CookTorranceLighting::preAddToRenderState(const RenderState* renderState, P
 
     mLightCount = renderState->getLightCount();
 
+    if(renderState->haveAreaLights())
+        mLtcLUT1SamplerIndex = ensureLtcLUTPresent(dstPass);
+
     if(mMetalRoughnessMapName.empty())
         return true;
 
