@@ -15,7 +15,7 @@
 // }
 
 /// configuration
-// use macros that will be default with Ogre 14
+// use macros that will be default with Ogre 15
 // #define USE_OGRE_FROM_FUTURE
 
 // @public-api
@@ -43,10 +43,20 @@
 vec4 texture2DLod(sampler2D s, vec2 v, float lod) { return tex2Dlod(s, vec4(v.x, v.y, 0, lod)); }
 
 #define samplerCube samplerCUBE
+vec4 textureCubeLod(samplerCube s, vec3 v, float lod) { return texCUBElod(s, vec4(v.x, v.y, v.z, lod)); }
+
 #define sampler2DShadow Sampler2DShadow
 
 #define mix lerp
 #define fract frac
+#define inversesqrt rsqrt
+#define dFdx ddx
+#define dFdy ddy
+
+float mod(float _a, float _b) { return _a - _b * floor(_a / _b); }
+vec2  mod(vec2  _a, vec2  _b) { return _a - _b * floor(_a / _b); }
+vec3  mod(vec3  _a, vec3  _b) { return _a - _b * floor(_a / _b); }
+vec4  mod(vec4  _a, vec4  _b) { return _a - _b * floor(_a / _b); }
 
 vec2 vec2_splat(float x) { return vec2(x, x); }
 vec3 vec3_splat(float x) { return vec3(x, x, x); }
@@ -94,9 +104,7 @@ mat3 mtxFromCols(vec3 a, vec3 b, vec3 c)
 // GLSL
 #include "GLSL_GL3Support.glsl"
 
-#ifndef USE_OGRE_FROM_FUTURE
-#define _UNIFORM_BINDING(b)
-#elif defined(VULKAN)
+#ifdef VULKAN
 #define _UNIFORM_BINDING(b) layout(binding = b + 2) uniform
 #elif __VERSION__ >= 420
 #define _UNIFORM_BINDING(b) layout(binding = b) uniform
@@ -148,6 +156,7 @@ mat3 mtxFromCols(vec3 a, vec3 b, vec3 c)
 #define COLOR0      3
 #define COLOR1      4
 #define COLOR COLOR0
+#define FOG         5
 #define BLENDINDICES 7
 #define TEXCOORD0   8
 #define TEXCOORD1   9
@@ -161,3 +170,16 @@ mat3 mtxFromCols(vec3 a, vec3 b, vec3 c)
 #endif
 
 #define OGRE_UNIFORMS(params) OGRE_UNIFORMS_BEGIN params OGRE_UNIFORMS_END
+
+// GL_EXT_shader_explicit_arithmetic_types polyfill
+#ifdef OGRE_GLSLES
+#define float32_t highp float
+#define f32vec2 highp vec2
+#define f32vec3 highp vec3
+#define f32vec4 highp vec4
+#else
+#define float32_t float
+#define f32vec2 vec2
+#define f32vec3 vec3
+#define f32vec4 vec4
+#endif
