@@ -49,12 +49,12 @@ namespace Ogre {
     /** This utility class is used to hold the information used to generate the matrices
     and other information required to automatically populate GpuProgramParameters.
 
-        This class exercises a lazy-update scheme in order to avoid having to update all
-        the information a GpuProgramParameters class could possibly want all the time. 
-        It relies on the SceneManager to update it when the base data has changed, and
-        will calculate concatenated matrices etc only when required, passing back precalculated
-        matrices when they are requested more than once when the underlying information has
-        not altered.
+    This class exercises a lazy-update scheme in order to avoid having to update all
+    the information a GpuProgramParameters class could possibly want all the time.
+    It relies on the SceneManager to update it when the base data has changed, and
+    will calculate concatenated matrices etc only when required, passing back precalculated
+    matrices when they are requested more than once when the underlying information has
+    not altered.
     */
     class _OgreExport AutoParamDataSource : public SceneMgtAlloc
     {
@@ -190,7 +190,7 @@ namespace Ogre {
         Vector4 getLightAs4DVector(size_t index) const;
         Vector3 getLightDirection(size_t index) const;
         Real getLightPowerScale(size_t index) const;
-        const Vector4f& getLightAttenuation(size_t index) const;
+        Vector4f getLightAttenuation(size_t index) const;
         Vector4f getSpotlightParams(size_t index) const;
         void setAmbientLightColour(const ColourValue& ambient);
         const ColourValue& getAmbientLightColour(void) const;
@@ -260,10 +260,11 @@ namespace Ogre {
         Vector3 getViewDirection() const;
         Vector3 getViewSideVector() const;
         Vector3 getViewUpVector() const;
-        Real getFOV() const;
-        Real getNearClipDistance() const;
-        Real getFarClipDistance() const;
+        float getFOV() const;
+        float getNearClipDistance() const;
+        float getFarClipDistance() const;
         int getPassNumber(void) const;
+        int getMaterialLodIndex() const;
         void setPassNumber(const int passNumber);
         void incPassNumber(void);
         void updateLightCustomGpuParameter(const GpuProgramParameters::AutoConstantEntry& constantEntry, GpuProgramParameters *params) const;
