@@ -33,7 +33,6 @@ THE SOFTWARE.
 #include "OgreCompositionTargetPass.h"
 #include "OgreCompositionPass.h"
 #include "OgreCompositorManager.h"
-#include "OgreRenderTarget.h"
 
 namespace Ogre {
 CompositorChain::CompositorChain(Viewport *vp):
@@ -122,7 +121,7 @@ void CompositorChain::createOriginalScene()
         /// Render everything, including skies
         pass = tp->createPass(CompositionPass::PT_RENDERSCENE);
         pass->setFirstRenderQueue(RENDER_QUEUE_BACKGROUND);
-        pass->setLastRenderQueue(RENDER_QUEUE_SKIES_LATE);
+        pass->setLastRenderQueue(RENDER_QUEUE_TRANSPARENTS);
         scene->load();
     }
     mOriginalScene = OGRE_NEW CompositorInstance(scene->getSupportedTechnique(), this);
@@ -139,7 +138,7 @@ void CompositorChain::destroyOriginalScene()
 }
 
 //-----------------------------------------------------------------------
-CompositorInstance* CompositorChain::addCompositor(CompositorPtr filter, size_t addPosition, const String& scheme)
+CompositorInstance* CompositorChain::addCompositor(const CompositorPtr& filter, size_t addPosition, const String& scheme)
 {
 
 
@@ -566,7 +565,7 @@ void CompositorChain::_notifyViewport(Viewport* vp)
 }
 //-----------------------------------------------------------------------
 void CompositorChain::RQListener::renderQueueStarted(uint8 id, 
-    const String& invocation, bool& skipThisQueue)
+    const String& cameraName, bool& skipThisQueue)
 {
     // Skip when not matching viewport
     // shadows update is nested within main viewport update
@@ -582,11 +581,6 @@ void CompositorChain::RQListener::renderQueueStarted(uint8 id,
     }
 }
 //-----------------------------------------------------------------------
-void CompositorChain::RQListener::renderQueueEnded(uint8 id, 
-    const String& invocation, bool& repeatThisQueue)
-{
-}
-//-----------------------------------------------------------------------
 void CompositorChain::RQListener::setOperation(CompositorInstance::TargetOperation *op,SceneManager *sm,RenderSystem *rs)
 {
     mOperation = op;
