@@ -59,7 +59,7 @@ namespace Ogre {
         mRenderOp.vertexData->vertexStart = 0; 
         mRenderOp.operationType = RenderOperation::OT_TRIANGLE_STRIP; 
         mRenderOp.useIndexes = false; 
-        mRenderOp.useGlobalInstancingVertexBufferIsAvailable = false;
+        mRenderOp.useGlobalInstancing = false;
 
         VertexDeclaration* decl = mRenderOp.vertexData->vertexDeclaration;
         VertexBufferBinding* bind = mRenderOp.vertexData->vertexBufferBinding;
@@ -202,10 +202,10 @@ namespace Ogre {
 
     const String& Rectangle2D::getMovableType() const
     {
-        return Rectangle2DFactory::FACTORY_TYPE_NAME;
+        return MOT_RECTANGLE2D;
     }
 
-    const String Rectangle2DFactory::FACTORY_TYPE_NAME = "Rectangle2D";
+    const String MOT_RECTANGLE2D = "Rectangle2D";
 
     MovableObject* Rectangle2DFactory::createInstanceImpl(const String& name, const NameValuePairList* params)
     {
