@@ -32,8 +32,6 @@ THE SOFTWARE.
 //  being aware of the 3D API. However there are a few
 //  simple functions which can have a base implementation
 
-#include "OgreRenderTarget.h"
-#include "OgreDepthBuffer.h"
 #include "OgreHardwareOcclusionQuery.h"
 #include "OgreComponents.h"
 
@@ -67,14 +65,6 @@ namespace Ogre {
         , mDerivedDepthBiasBase(0.0f)
         , mDerivedDepthBiasMultiplier(0.0f)
         , mDerivedDepthBiasSlopeScale(0.0f)
-        , mGlobalInstanceVertexBufferVertexDeclaration(NULL)
-        , mGlobalNumberOfInstances(1)
-        , mVertexProgramBound(false)
-        , mGeometryProgramBound(false)
-        , mFragmentProgramBound(false)
-        , mTessellationHullProgramBound(false)
-        , mTessellationDomainProgramBound(false)
-        , mComputeProgramBound(false)
         , mClipPlanesDirty(true)
         , mRealCapabilities(0)
         , mCurrentCapabilities(0)
@@ -82,6 +72,8 @@ namespace Ogre {
         , mNativeShadingLanguageVersion(0)
         , mTexProjRelative(false)
         , mTexProjRelativeOrigin(Vector3::ZERO)
+        , mGlobalInstanceVertexDeclaration(NULL)
+        , mGlobalNumberOfInstances(1)
     {
         mEventNames.push_back("RenderSystemCapabilitiesCreated");
     }
@@ -144,6 +136,18 @@ namespace Ogre {
         mFixedFunctionParams->setAutoConstant(light_offset + 5, GpuProgramParameters::ACT_SPOTLIGHT_PARAMS, index);
     }
 
+    const HardwareBufferPtr& RenderSystem::updateDefaultUniformBuffer(GpuProgramType gptype, const ConstantList& params)
+    {
+        auto& ubo = mUniformBuffer[gptype];
+        if (!ubo || ubo->getSizeInBytes() < params.size())
+        {
+            ubo = HardwareBufferManager::getSingleton().createUniformBuffer(params.size());
+        }
+
+        ubo->writeData(0, params.size(), params.data(), true);
+
+        return ubo;
+    }
     //-----------------------------------------------------------------------
     RenderSystem::~RenderSystem()
     {
@@ -231,7 +235,7 @@ namespace Ogre {
         }
 
 #if OGRE_NO_QUAD_BUFFER_STEREO == 0
-        if((opt = mOptions.find("Stereo Mode")) != end)
+        if((opt = mOptions.find("Frame Sequential Stereo")) != end)
             miscParams["stereoMode"] = opt->second.currentValue;
 #endif
         return ret;
@@ -286,12 +290,7 @@ namespace Ogre {
         // They should ALL call this superclass method from
         //   their own initialise() implementations.
         
-        mVertexProgramBound = false;
-        mGeometryProgramBound = false;
-        mFragmentProgramBound = false;
-        mTessellationHullProgramBound = false;
-        mTessellationDomainProgramBound = false;
-        mComputeProgramBound = false;
+        mProgramBound.fill(false);
     }
 
     //---------------------------------------------------------------------------------------------
@@ -455,8 +454,7 @@ namespace Ogre {
         auto calcMode = tl._deriveTexCoordCalcMethod();
         if(calcMode == TEXCALC_PROJECTIVE_TEXTURE)
         {
-            auto frustum = tl.getEffects().find(TextureUnitState::ET_PROJECTIVE_TEXTURE)->second.frustum;
-            _setTextureCoordCalculation(texUnit, calcMode, frustum);
+            _setTextureCoordCalculation(texUnit, calcMode, tl.getProjectiveTexturingFrustum());
         }
         else
         {
@@ -467,15 +465,6 @@ namespace Ogre {
         _setTextureMatrix(texUnit, tl.getTextureTransform());
     }
     //-----------------------------------------------------------------------
-    void RenderSystem::_setVertexTexture(size_t unit, const TexturePtr& tex)
-    {
-        OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, 
-            "This rendersystem does not support separate vertex texture samplers, "
-            "you should use the regular texture samplers which are shared between "
-            "the vertex and fragment units.", 
-            "RenderSystem::_setVertexTexture");
-    }
-    //-----------------------------------------------------------------------
     void RenderSystem::_disableTextureUnit(size_t texUnit)
     {
         _setTexture(texUnit, false, sNullTexPtr);
@@ -492,16 +481,6 @@ namespace Ogre {
             _disableTextureUnit(i);
         }
     }
-    //-----------------------------------------------------------------------
-    void RenderSystem::_setTextureUnitFiltering(size_t unit, FilterOptions minFilter,
-            FilterOptions magFilter, FilterOptions mipFilter)
-    {
-        OGRE_IGNORE_DEPRECATED_BEGIN
-        _setTextureUnitFiltering(unit, FT_MIN, minFilter);
-        _setTextureUnitFiltering(unit, FT_MAG, magFilter);
-        _setTextureUnitFiltering(unit, FT_MIP, mipFilter);
-        OGRE_IGNORE_DEPRECATED_END
-    }
     //---------------------------------------------------------------------
     void RenderSystem::_cleanupDepthBuffers( bool bCleanManualBuffers )
     {
@@ -586,14 +565,20 @@ namespace Ogre {
         // Remove all the render targets. Destroy primary target last since others may depend on it.
         // Keep mRenderTargets valid all the time, so that render targets could receive
         // appropriate notifications, for example FBO based about GL context destruction.
-        RenderTarget* primary {nullptr};
-        for (auto &&a : mRenderTargets) {
-            if (!primary && a.second->isPrimary()) {
-                primary = a.second;
-                continue;
+        RenderTarget* primary = 0;
+        for (RenderTargetMap::iterator it = mRenderTargets.begin(); it != mRenderTargets.end(); /* note - no increment */)
+        {
+            RenderTarget* current = it->second;
+            if (!primary && current->isPrimary())
+            {
+                ++it;
+                primary = current;
+            }
+            else
+            {
+                it = mRenderTargets.erase(it);
+                OGRE_DELETE current;
             }
-            OGRE_DELETE a.second;
-            mRenderTargets.erase(a.first);
         }
         OGRE_DELETE primary;
         mRenderTargets.clear();
@@ -734,36 +719,14 @@ namespace Ogre {
 
         const uint16 mask = GPV_PASS_ITERATION_NUMBER;
 
-        if (mActiveVertexGpuProgramParameters)
+        for (int i = 0; i < GPT_COUNT; i++)
         {
-            mActiveVertexGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_VERTEX_PROGRAM, mActiveVertexGpuProgramParameters, mask);
-        }
-        if (mActiveGeometryGpuProgramParameters)
-        {
-            mActiveGeometryGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_GEOMETRY_PROGRAM, mActiveGeometryGpuProgramParameters, mask);
-        }
-        if (mActiveFragmentGpuProgramParameters)
-        {
-            mActiveFragmentGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_FRAGMENT_PROGRAM, mActiveFragmentGpuProgramParameters, mask);
-        }
-        if (mActiveTessellationHullGpuProgramParameters)
-        {
-            mActiveTessellationHullGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_HULL_PROGRAM, mActiveTessellationHullGpuProgramParameters, mask);
-        }
-        if (mActiveTessellationDomainGpuProgramParameters)
-        {
-            mActiveTessellationDomainGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_DOMAIN_PROGRAM, mActiveTessellationDomainGpuProgramParameters, mask);
-        }
-        if (mActiveComputeGpuProgramParameters)
-        {
-            mActiveComputeGpuProgramParameters->incPassIterationNumber();
-            bindGpuProgramParameters(GPT_COMPUTE_PROGRAM, mActiveComputeGpuProgramParameters, mask);
+            if (!mActiveParameters[i])
+                continue;
+            mActiveParameters[i]->incPassIterationNumber();
+            bindGpuProgramParameters(GpuProgramType(i), mActiveParameters[i], mask);
         }
+
         return true;
     }
 
@@ -813,80 +776,26 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void RenderSystem::bindGpuProgram(GpuProgram* prg)
     {
-        switch(prg->getType())
-        {
-        case GPT_VERTEX_PROGRAM:
-            // mark clip planes dirty if changed (programmable can change space)
-            if (!mVertexProgramBound && !mClipPlanes.empty())
-                mClipPlanesDirty = true;
+        auto gptype = prg->getType();
+        // mark clip planes dirty if changed (programmable can change space)
+        if(gptype == GPT_VERTEX_PROGRAM && !mClipPlanes.empty() && !mProgramBound[gptype])
+            mClipPlanesDirty = true;
 
-            mVertexProgramBound = true;
-            break;
-        case GPT_GEOMETRY_PROGRAM:
-            mGeometryProgramBound = true;
-            break;
-        case GPT_FRAGMENT_PROGRAM:
-            mFragmentProgramBound = true;
-            break;
-        case GPT_HULL_PROGRAM:
-            mTessellationHullProgramBound = true;
-            break;
-        case GPT_DOMAIN_PROGRAM:
-            mTessellationDomainProgramBound = true;
-            break;
-        case GPT_COMPUTE_PROGRAM:
-            mComputeProgramBound = true;
-            break;
-        }
+        mProgramBound[gptype] = true;
     }
     //-----------------------------------------------------------------------
     void RenderSystem::unbindGpuProgram(GpuProgramType gptype)
     {
-        switch(gptype)
-        {
-        case GPT_VERTEX_PROGRAM:
-            // mark clip planes dirty if changed (programmable can change space)
-            if (mVertexProgramBound && !mClipPlanes.empty())
-                mClipPlanesDirty = true;
-            mVertexProgramBound = false;
-            break;
-        case GPT_GEOMETRY_PROGRAM:
-            mGeometryProgramBound = false;
-            break;
-        case GPT_FRAGMENT_PROGRAM:
-            mFragmentProgramBound = false;
-            break;
-        case GPT_HULL_PROGRAM:
-            mTessellationHullProgramBound = false;
-            break;
-        case GPT_DOMAIN_PROGRAM:
-            mTessellationDomainProgramBound = false;
-            break;
-        case GPT_COMPUTE_PROGRAM:
-            mComputeProgramBound = false;
-            break;
-        }
+        // mark clip planes dirty if changed (programmable can change space)
+        if(gptype == GPT_VERTEX_PROGRAM && !mClipPlanes.empty() && mProgramBound[gptype])
+            mClipPlanesDirty = true;
+
+        mProgramBound[gptype] = false;
     }
     //-----------------------------------------------------------------------
     bool RenderSystem::isGpuProgramBound(GpuProgramType gptype)
     {
-        switch(gptype)
-        {
-        case GPT_VERTEX_PROGRAM:
-            return mVertexProgramBound;
-        case GPT_GEOMETRY_PROGRAM:
-            return mGeometryProgramBound;
-        case GPT_FRAGMENT_PROGRAM:
-            return mFragmentProgramBound;
-        case GPT_HULL_PROGRAM:
-            return mTessellationHullProgramBound;
-        case GPT_DOMAIN_PROGRAM:
-            return mTessellationDomainProgramBound;
-        case GPT_COMPUTE_PROGRAM:
-            return mComputeProgramBound;
-        }
-        // Make compiler happy
-        return false;
+        return mProgramBound[gptype];
     }
     //---------------------------------------------------------------------
     void RenderSystem::_setTextureProjectionRelativeTo(bool enabled, const Vector3& pos)
@@ -896,18 +805,6 @@ namespace Ogre {
 
     }
     //---------------------------------------------------------------------
-    RenderSystem::RenderSystemContext* RenderSystem::_pauseFrame(void)
-    {
-        _endFrame();
-        return new RenderSystem::RenderSystemContext;
-    }
-    //---------------------------------------------------------------------
-    void RenderSystem::_resumeFrame(RenderSystemContext* context)
-    {
-        _beginFrame();
-        delete context;
-    }
-    //---------------------------------------------------------------------
     const String& RenderSystem::_getDefaultViewportMaterialScheme( void ) const
     {
 #ifdef RTSHADER_SYSTEM_BUILD_CORE_SHADERS
@@ -919,42 +816,12 @@ namespace Ogre {
         return MSN_DEFAULT;
     }
     //---------------------------------------------------------------------
-    Ogre::HardwareVertexBufferSharedPtr RenderSystem::getGlobalInstanceVertexBuffer() const
-    {
-        return mGlobalInstanceVertexBuffer;
-    }
-    //---------------------------------------------------------------------
     void RenderSystem::setGlobalInstanceVertexBuffer( const HardwareVertexBufferSharedPtr &val )
     {
-        if ( val && !val->isInstanceData() )
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
-                        "A none instance data vertex buffer was set to be the global instance vertex buffer.",
-                        "RenderSystem::setGlobalInstanceVertexBuffer");
-        }
+        OgreAssert(!val || val->isInstanceData(), "not an instance buffer");
         mGlobalInstanceVertexBuffer = val;
     }
     //---------------------------------------------------------------------
-    size_t RenderSystem::getGlobalNumberOfInstances() const
-    {
-        return mGlobalNumberOfInstances;
-    }
-    //---------------------------------------------------------------------
-    void RenderSystem::setGlobalNumberOfInstances( const size_t val )
-    {
-        mGlobalNumberOfInstances = val;
-    }
-
-    VertexDeclaration* RenderSystem::getGlobalInstanceVertexBufferVertexDeclaration() const
-    {
-        return mGlobalInstanceVertexBufferVertexDeclaration;
-    }
-    //---------------------------------------------------------------------
-    void RenderSystem::setGlobalInstanceVertexBufferVertexDeclaration( VertexDeclaration* val )
-    {
-        mGlobalInstanceVertexBufferVertexDeclaration = val;
-    }
-    //---------------------------------------------------------------------
     void RenderSystem::getCustomAttribute(const String& name, void* pData)
     {
         OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Attribute not found.", "RenderSystem::getCustomAttribute");
@@ -1009,9 +876,9 @@ namespace Ogre {
 
 #if OGRE_NO_QUAD_BUFFER_STEREO == 0
         ConfigOption optStereoMode;
-        optStereoMode.name = "Stereo Mode";
-        optStereoMode.possibleValues.push_back(StringConverter::toString(SMT_NONE));
-        optStereoMode.possibleValues.push_back(StringConverter::toString(SMT_FRAME_SEQUENTIAL));
+        optStereoMode.name = "Frame Sequential Stereo";
+        optStereoMode.possibleValues.push_back("Off");
+        optStereoMode.possibleValues.push_back("On");
         optStereoMode.currentValue = optStereoMode.possibleValues[0];
         optStereoMode.immutable = false;
 
