@@ -338,6 +338,8 @@ void DotSceneLoader::processLight(pugi::xml_node& XMLNode, SceneNode* pParent)
         pLight->setType(Light::LT_SPOTLIGHT);
     else if (sValue == "radPoint")
         pLight->setType(Light::LT_POINT);
+    else if (sValue == "rect")
+        pLight->setType(Light::LT_RECTLIGHT);
 
     pLight->setVisible(getAttribBool(XMLNode, "visible", true));
     pLight->setCastShadows(getAttribBool(XMLNode, "castShadows", true));
@@ -361,6 +363,14 @@ void DotSceneLoader::processLight(pugi::xml_node& XMLNode, SceneNode* pParent)
         if (auto pElement = XMLNode.child("lightAttenuation"))
             processLightAttenuation(pElement, pLight);
     }
+
+    if (sValue == "rect")
+    {
+        // Process lightSourceSize (?)
+        if (auto pElement = XMLNode.child("lightSourceSize"))
+            processLightSourceSize(pElement, pLight);
+	}
+
     // Process userDataReference (?)
     if (auto pElement = XMLNode.child("userData"))
         processUserData(pElement, pLight->getUserObjectBindings());
@@ -765,9 +775,6 @@ void DotSceneLoader::processSkyBox(pugi::xml_node& XMLNode)
     String material = getAttrib(XMLNode, "material", "BaseWhite");
     Real distance = getAttribReal(XMLNode, "distance", 5000);
     bool drawFirst = getAttribBool(XMLNode, "drawFirst", true);
-    bool active = getAttribBool(XMLNode, "active", false);
-    if (!active)
-        return;
 
     // Process rotation (?)
     Quaternion rotation = Quaternion::IDENTITY;
@@ -847,6 +854,16 @@ void DotSceneLoader::processLightAttenuation(pugi::xml_node& XMLNode, Light* pLi
     pLight->setAttenuation(range, constant, linear, quadratic);
 }
 
+void DotSceneLoader::processLightSourceSize(pugi::xml_node& XMLNode, Light* pLight)
+{
+    // Process attributes
+    Real width = getAttribReal(XMLNode, "width");
+    Real height = getAttribReal(XMLNode, "height");
+
+    // Setup the light range
+    pLight->setSourceSize(width, height);
+}
+
 void DotSceneLoader::processUserData(pugi::xml_node& XMLNode, UserObjectBindings& userData)
 {
     // Process node (*)
@@ -1050,6 +1067,9 @@ void DotSceneLoader::writeNode(pugi::xml_node& parentXML, const SceneNode* n)
             case Light::LT_SPOTLIGHT:
                 light.append_attribute("type") = "spot";
                 break;
+            case Light::LT_RECTLIGHT:
+                light.append_attribute("type") = "rect";
+                break;
             }
 
             if(l->getType() != Light::LT_DIRECTIONAL)
