@@ -74,7 +74,7 @@ namespace Ogre {
         stbi_convert_iphone_png_to_rgb(1);
         stbi_set_unpremultiply_on_load(1);
 
-        LogManager::getSingleton().logMessage("stb_image - v2.28 - public domain image loader");
+        LogManager::getSingleton().logMessage("stb_image - v2.30 - public domain image loader");
         
         // Register codecs
         String exts = "jpeg,jpg,png,bmp,psd,tga,gif,pic,ppm,pgm,hdr";
@@ -214,11 +214,6 @@ namespace Ogre {
         return BLANKSTRING;
     }
 
-    const String& STBIPlugin::getName() const {
-        static String name = "STB Image Codec";
-        return name;
-    }
-
 #ifndef OGRE_STATIC_LIB
     extern "C" void _OgreSTBICodecExport dllStartPlugin();
     extern "C" void _OgreSTBICodecExport dllStopPlugin();
