@@ -479,12 +479,6 @@ namespace Ogre
         return mXDisplay;
     }
 
-    //-------------------------------------------------------------------------------------------------//
-    String GLXGLSupport::getDisplayName(void)
-    {
-        return String((const char*)XDisplayName(DisplayString(mGLDisplay)));
-    }
-
     //-------------------------------------------------------------------------------------------------//
     GLXFBConfig* GLXGLSupport::chooseFBConfig(const GLint *attribList, GLint *nElements)
     {
