@@ -48,8 +48,8 @@ namespace Ogre {
         //There are up to 8 VES_TEXTURE_COORDINATES. Occupy range [8; 16)
         //Range [14; 16) overlaps with VES_TANGENT & VES_BINORMAL
         //(slot 16 is where const buffers start)
-        14,// VES_BINORMAL - 1
-        15,  // VES_TANGENT - 1
+        15,// VES_BINORMAL - 1
+        14,  // VES_TANGENT - 1
     };
 
     uint32 MetalProgram::getAttributeIndex(VertexElementSemantic semantic)
@@ -69,8 +69,7 @@ namespace Ogre {
         mLibrary( nil ),
         mFunction( nil ),
         mDevice( device ),
-        mCompiled( false ),
-        mConstantsBytesToWrite( 0 )
+        mCompiled( false )
     {
         if (createParamDictionary("MetalProgram"))
         {
@@ -85,6 +84,7 @@ namespace Ogre {
                                             PT_STRING),&msCmdShaderReflectionPairHint);
         }
         mTargetBufferName = "";
+        mEntryPoint = "mainfunc"; // 1.4.4: A Metal function cannot be called main.
     }
     //---------------------------------------------------------------------------
     MetalProgram::~MetalProgram()
@@ -130,19 +130,6 @@ namespace Ogre {
         NSMutableDictionary<NSString *, NSObject *> *preprocessorMacros =
                 [NSMutableDictionary dictionary];
 
-        preprocessorMacros[@"CONST_SLOT_START"] =
-                [NSNumber numberWithUnsignedInt:mType != GPT_COMPUTE_PROGRAM ?
-                    OGRE_METAL_CONST_SLOT_START : OGRE_METAL_CS_CONST_SLOT_START];
-        preprocessorMacros[@"TEX_SLOT_START"] =
-                [NSNumber numberWithUnsignedInt:mType != GPT_COMPUTE_PROGRAM ?
-                    OGRE_METAL_TEX_SLOT_START : OGRE_METAL_CS_TEX_SLOT_START];
-        preprocessorMacros[@"UAV_SLOT_START"] =
-                [NSNumber numberWithUnsignedInt:mType != GPT_COMPUTE_PROGRAM ?
-                    OGRE_METAL_UAV_SLOT_START : OGRE_METAL_CS_UAV_SLOT_START];
-        preprocessorMacros[@"PARAMETER_SLOT"] =
-                [NSNumber numberWithUnsignedInt:mType != GPT_COMPUTE_PROGRAM ?
-                    OGRE_METAL_PARAMETER_SLOT : OGRE_METAL_CS_PARAMETER_SLOT];
-
         parsePreprocessorDefinitions( preprocessorMacros );
 
         options.preprocessorMacros = preprocessorMacros;
@@ -259,7 +246,7 @@ namespace Ogre {
 
             for( MTLArgument *arg in arguments )
             {
-                if( arg.type == MTLArgumentTypeBuffer && arg.index == OGRE_METAL_CS_PARAMETER_SLOT )
+                if( arg.type == MTLArgumentTypeBuffer && arg.index >= UNIFORM_INDEX_START )
                     analyzeParameterBuffer( arg );
             }
         }
@@ -298,6 +285,10 @@ namespace Ogre {
                              "MetalProgram::analyzeRenderParameters" );
             }
             shader->load();
+
+            if(shader->hasCompileError())
+                return;
+
             assert( dynamic_cast<MetalProgram*>( shader->_getBindingDelegate() ) );
             MetalProgram *vertexShader = static_cast<MetalProgram*>( shader->_getBindingDelegate() );
             autoFillDummyVertexAttributesForShader( vertexShader->getMetalFunction(), psd );
@@ -336,7 +327,7 @@ namespace Ogre {
 
             for( MTLArgument *arg in arguments )
             {
-                if( arg.type == MTLArgumentTypeBuffer && arg.index == OGRE_METAL_CONST_SLOT_START )
+                if( arg.type == MTLArgumentTypeBuffer && arg.index >= UNIFORM_INDEX_START )
                     analyzeParameterBuffer( arg );
             }
         }
@@ -372,12 +363,6 @@ namespace Ogre {
                 String varName = arg.name.UTF8String;
 
                 mConstantDefs->map.insert( GpuConstantDefinitionMap::value_type( varName, def ) );
-                mConstantDefsSorted.push_back( def );
-
-                mConstantsBytesToWrite = std::max<uint32>( mConstantsBytesToWrite,
-                                                           def.logicalIndex +
-                                                           def.arraySize * def.elementSize *
-                                                           sizeof(float) );
             }
         }
 
@@ -419,12 +404,6 @@ namespace Ogre {
                 String varName = member.name.UTF8String;
 
                 mConstantDefs->map.insert( GpuConstantDefinitionMap::value_type( varName, def ) );
-                mConstantDefsSorted.push_back( def );
-
-                mConstantsBytesToWrite = std::max<uint32>( mConstantsBytesToWrite,
-                                                           def.logicalIndex +
-                                                           def.arraySize * def.elementSize *
-                                                           sizeof(float) );
             }
         }
     }
@@ -456,41 +435,16 @@ namespace Ogre {
 
         if( mType != GPT_COMPUTE_PROGRAM )
         {
-            //You think this is a code smell? How about making BUILDconstantDefinitions const???
-            //It's an oxymoron.
-            const_cast<MetalProgram*>(this)->analyzeRenderParameters();
+            analyzeRenderParameters();
         }
         else
         {
-            const_cast<MetalProgram*>(this)->analyzeComputeParameters();
+            analyzeComputeParameters();
         }
 
         mLogicalToPhysical.reset(); // disallow access by index for now
     }
     //-----------------------------------------------------------------------
-    uint32 MetalProgram::getBufferRequiredSize(void) const
-    {
-        return mConstantsBytesToWrite;
-    }
-    //-----------------------------------------------------------------------
-    void MetalProgram::updateBuffers( const GpuProgramParametersPtr &params,
-                                      uint8 * RESTRICT_ALIAS dstData )
-    {
-        auto itor = mConstantDefsSorted.begin();
-        auto end  = mConstantDefsSorted.end();
-
-        while( itor != end )
-        {
-            const GpuConstantDefinition& def = *itor;
-
-            const void * RESTRICT_ALIAS src = params->getConstantList().data() + def.physicalIndex;
-
-            memcpy( &dstData[def.logicalIndex], src, def.elementSize * def.arraySize * sizeof(float) );
-
-            ++itor;
-        }
-    }
-    //-----------------------------------------------------------------------
     String MetalProgram::CmdShaderReflectionPairHint::doGet(const void *target) const
     {
         return static_cast<const MetalProgram*>(target)->getShaderReflectionPairHint();
