@@ -52,8 +52,6 @@ THE SOFTWARE.
 #include "OgreVulkanWindow.h"
 #include "OgrePixelFormat.h"
 
-#define USE_VALIDATION_LAYERS 0
-
 namespace Ogre
 {
     static const uint32 VERTEX_ATTRIBUTE_INDEX[VES_COUNT] =
@@ -534,11 +532,6 @@ namespace Ogre
                 rsc->setCapability( RSC_TEXTURE_COMPRESSION_ETC2 );
             }
 
-            vkGetPhysicalDeviceFormatProperties( mDevice->mPhysicalDevice,
-                                                 VulkanMappings::get( PF_PVRTC_RGB2 ), &props );
-            if( props.optimalTilingFeatures & VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT )
-                rsc->setCapability( RSC_TEXTURE_COMPRESSION_PVRTC );
-
             vkGetPhysicalDeviceFormatProperties(
                 mDevice->mPhysicalDevice, VulkanMappings::get( PF_ASTC_RGBA_4X4_LDR ), &props );
             if( props.optimalTilingFeatures & VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT )
@@ -560,7 +553,7 @@ namespace Ogre
         //rsc->setCapability( RSC_STORE_AND_MULTISAMPLE_RESOLVE );
         //rsc->setCapability( RSC_TEXTURE_GATHER );
 
-        rsc->setCapability( RSC_COMPUTE_PROGRAM );
+        rsc->setCapability( RSC_VERTEX_PROGRAM );
         //rsc->setCapability( RSC_UAV );
         //rsc->setCapability( RSC_TYPED_UAV_LOADS );
         //rsc->setCapability( RSC_EXPLICIT_FSAA_RESOLVE );
@@ -580,14 +573,17 @@ namespace Ogre
         rsc->setNonPOW2TexturesLimited( false );
         rsc->setCapability( RSC_HWRENDER_TO_TEXTURE );
         rsc->setCapability( RSC_TEXTURE_FLOAT );
-        rsc->setCapability( RSC_POINT_SPRITES );
-        rsc->setCapability( RSC_POINT_EXTENDED_PARAMETERS );
+        if( mActiveDevice->mDeviceFeatures.largePoints )
+        {
+            rsc->setCapability( RSC_POINT_SPRITES );
+        }
         rsc->setCapability( RSC_TEXTURE_2D_ARRAY );
         rsc->setCapability( RSC_ALPHA_TO_COVERAGE );
         rsc->setCapability( RSC_HW_GAMMA );
         rsc->setCapability( RSC_VERTEX_BUFFER_INSTANCE_DATA );
         rsc->setCapability(RSC_VERTEX_FORMAT_INT_10_10_10_2);
-        rsc->setMaxPointSize( 256 );
+        rsc->setCapability(RSC_VERTEX_FORMAT_16X3);
+        rsc->setMaxPointSize( deviceLimits.pointSizeRange[1] );
 
         //rsc->setMaximumResolutions( 16384, 4096, 16384 );
         auto maxFloatVectors = deviceLimits.maxUniformBufferRange / (4 * sizeof(float));
@@ -764,6 +760,16 @@ namespace Ogre
                     }
                     else if( extensionName == VK_EXT_SHADER_SUBGROUP_VOTE_EXTENSION_NAME )
                         deviceExtensions.push_back( VK_EXT_SHADER_SUBGROUP_VOTE_EXTENSION_NAME );
+#ifdef VK_EXT_mesh_shader
+                    else if( extensionName == VK_EXT_MESH_SHADER_EXTENSION_NAME)
+                    {
+                        deviceExtensions.push_back( VK_EXT_MESH_SHADER_EXTENSION_NAME );
+                        deviceExtensions.push_back( VK_KHR_SPIRV_1_4_EXTENSION_NAME );
+                        mRealCapabilities->setCapability(RSC_MESH_PROGRAM);
+
+                        mDescriptorSetBindings[0].stageFlags |= VK_SHADER_STAGE_MESH_BIT_NV;
+                    }
+#endif
                 }
             }
 
@@ -1021,7 +1027,7 @@ namespace Ogre
         if(!vertexBuffers.empty())
             vkCmdBindVertexBuffers(cmdBuffer, 0, vertexBuffers.size(), vertexBuffers.data(), offsets);
 
-        if(op.indexData)
+        if(op.useIndexes)
         {
             auto itype = VkIndexType(op.indexData->indexBuffer->getType());
             auto b = op.indexData->indexBuffer->_getImpl<VulkanHardwareBuffer>()->getVkBuffer();
@@ -1036,7 +1042,14 @@ namespace Ogre
         vkCmdBindPipeline( cmdBuffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline );
 
         // Render to screen!
-        if( op.useIndexes )
+        if(mProgramBound[GPT_MESH_PROGRAM])
+        {
+#ifdef VK_EXT_mesh_shader
+            OgreAssert(op.indexData, "indexData required for mesh shader");
+            vkCmdDrawMeshTasksEXT(cmdBuffer, op.indexData->indexCount, 1, 1);
+#endif
+        }
+        else if( op.useIndexes )
         {
             do
             {
@@ -1078,41 +1091,25 @@ namespace Ogre
     void VulkanRenderSystem::bindGpuProgram(GpuProgram* prg)
     {
         auto shader = static_cast<VulkanProgram*>(prg);
-        shaderStages[prg->getType()] = shader->getPipelineShaderStageCi();
+        shaderStages[prg->getType() % GPT_PIPELINE_COUNT] = shader->getPipelineShaderStageCi();
         mBoundGpuPrograms[prg->getType()] = prg->_getHash();
+
+        RenderSystem::bindGpuProgram(prg);
     }
     void VulkanRenderSystem::bindGpuProgramParameters( GpuProgramType gptype,
                                                        const GpuProgramParametersPtr& params,
                                                        uint16 variabilityMask )
     {
-        switch( gptype )
-        {
-        case GPT_VERTEX_PROGRAM:
-            mActiveVertexGpuProgramParameters = params;
-            break;
-        case GPT_FRAGMENT_PROGRAM:
-            mActiveFragmentGpuProgramParameters = params;
-            break;
-        case GPT_GEOMETRY_PROGRAM:
-            mActiveGeometryGpuProgramParameters = params;
-            break;
-        case GPT_HULL_PROGRAM:
-            mActiveTessellationHullGpuProgramParameters = params;
-            break;
-        case GPT_DOMAIN_PROGRAM:
-            mActiveTessellationDomainGpuProgramParameters = params;
-            break;
-        case GPT_COMPUTE_PROGRAM:
-            mActiveComputeGpuProgramParameters = params;
-            break;
-        }
+        mActiveParameters[gptype] = params;
+
+        int dstUBO = gptype % GPT_PIPELINE_COUNT;
 
         auto sizeBytes = params->getConstantList().size();
-        if(sizeBytes && gptype <= GPT_FRAGMENT_PROGRAM)
+        if(sizeBytes && dstUBO < 2)
         {
             auto step =
                 alignToNextMultiple(sizeBytes, mDevice->mDeviceProperties.limits.minUniformBufferOffsetAlignment);
-            mUBOInfo[gptype].range = sizeBytes;
+            mUBOInfo[dstUBO].range = sizeBytes;
 
             if (std::accumulate(mAutoParamsBufferUsage.begin(), mAutoParamsBufferUsage.end(), 0) + step >=
                 mAutoParamsBuffer->getSizeInBytes())
@@ -1124,7 +1121,7 @@ namespace Ogre
             if((mAutoParamsBufferPos + sizeBytes) >= mAutoParamsBuffer->getSizeInBytes())
                 mAutoParamsBufferPos = 0;
 
-            mUBODynOffsets[gptype] = mAutoParamsBufferPos;
+            mUBODynOffsets[dstUBO] = mAutoParamsBufferPos;
 
             mAutoParamsBuffer->writeData(mAutoParamsBufferPos, sizeBytes, params->getConstantList().data());
             mAutoParamsBufferPos += step;
@@ -1261,7 +1258,12 @@ namespace Ogre
         }
     }
 
-    void VulkanRenderSystem::_setCullingMode(CullingMode mode) { rasterState.cullMode = VulkanMappings::get(mode); }
+    void VulkanRenderSystem::_setCullingMode(CullingMode mode)
+    {
+        rasterState.cullMode = VulkanMappings::get(mode);
+        // flipFrontFace result is inverted due to inverted clip space Y
+        rasterState.frontFace = !flipFrontFace() ? VK_FRONT_FACE_CLOCKWISE : VK_FRONT_FACE_COUNTER_CLOCKWISE;
+    }
 
     void VulkanRenderSystem::_setDepthBias(float constantBias, float slopeScaleBias)
     {
