@@ -36,11 +36,15 @@ THE SOFTWARE.
 #include <iostream>
 
 #include "Ogre.h"
+#include "OgreCodec.h"
 #include "OgreDefaultHardwareBufferManager.h"
 
-#include "OgreAssimpLoader.h"
 #include <assimp/postprocess.h>
 
+#ifdef OGRE_BUILD_COMPONENT_RTSHADERSYSTEM
+#include <OgreShaderGenerator.h>
+#endif
+
 using namespace Ogre;
 
 namespace
@@ -74,7 +78,7 @@ struct AssOptions
     String dest;
     String logFile;
 
-    AssimpLoader::Options options;
+    BinaryOptionList options;
 
     AssOptions() { logFile = "OgreAssimp.log"; };
 };
@@ -102,18 +106,18 @@ AssOptions parseArgs(int numArgs, char** args)
 
     if (unOpt["-q"])
     {
-        opts.options.params |= AssimpLoader::LP_QUIET_MODE;
+        opts.options["quiet"] = "true";
     }
     if (unOpt["-3ds_ani_fix"])
     {
-        opts.options.params |= AssimpLoader::LP_CUT_ANIMATION_WHERE_NO_FURTHER_CHANGE;
+        opts.options["cutAnimation"] = "true";
     }
 
-    opts.options.postProcessSteps = aiProcessPreset_TargetRealtime_Quality;
+    opts.options["postProcessSteps"] = std::to_string(aiProcessPreset_TargetRealtime_Quality);
     opts.logFile = binOpt["-log"];
-    StringConverter::parse(binOpt["-aniSpeedMod"], opts.options.animationSpeedModifier);
-    opts.options.customAnimationName = binOpt["-aniName"];
-    StringConverter::parse(binOpt["-max_edge_angle"], opts.options.maxEdgeAngle);
+    opts.options["animationSpeedModifier"] = binOpt["-aniSpeedMod"];
+    opts.options["customAnimationName"] = binOpt["-aniName"];
+    opts.options["maxEdgeAngle"] = binOpt["-max_edge_angle"];
 
     // Source / dest
     if (numArgs > startIndex)
@@ -147,7 +151,7 @@ AssOptions parseArgs(int numArgs, char** args)
 
         std::cout << "source file               = " << opts.source << std::endl;
         std::cout << "destination               = " << opts.dest << std::endl;
-        std::cout << "animation speed modifier  = " << opts.options.animationSpeedModifier << std::endl;
+        std::cout << "animation speed modifier  = " << opts.options["animationSpeedModifier"] << std::endl;
         std::cout << "log file                  = " << opts.logFile << std::endl;
 
         std::cout << "-- END OPTIONS --" << std::endl;
@@ -176,7 +180,16 @@ int main(int numargs, char** args)
 
     try
     {
+        FileSystemLayer fsLayer("Ogre3D");
+        ConfigFile pluginsCfg;
+        pluginsCfg.load(fsLayer.getConfigFilePath("plugins.cfg"));
+
+        auto pluginDir = pluginsCfg.getSetting("PluginFolder")+"/";
+
         logMgr.setDefaultLog(NULL); // swallow startup messages
+
+        DefaultHardwareBufferManager bufferManager; // needed because we don't have a rendersystem
+
         Root root("", "", "");
         // get rid of the temporary log as we use the new log now
         logMgr.destroyLog("Temporary log");
@@ -184,19 +197,24 @@ int main(int numargs, char** args)
         // use the log specified by the cmdline params
         logMgr.setDefaultLog(logMgr.createLog(opts.logFile, false, true));
 
+        root.loadPlugin(pluginDir + "/Codec_Assimp");
+
         MaterialManager::getSingleton().initialise();
 
-        DefaultHardwareBufferManager bufferManager; // needed because we don't have a rendersystem
+#ifdef OGRE_BUILD_COMPONENT_RTSHADERSYSTEM
+        RTShader::ShaderGenerator::initialize();
+#endif
+
         DefaultTextureManager texMgr;
 
         String basename, ext, path;
         StringUtil::splitFullFilename(opts.source, basename, ext, path);
 
-        MeshPtr mesh = MeshManager::getSingleton().createManual(basename + "." + ext, RGN_DEFAULT);
-        SkeletonPtr skeleton;
+        auto codec = Codec::getCodec(ext);
 
-        AssimpLoader loader;
-        loader.load(opts.source, mesh.get(), skeleton, opts.options);
+        MeshPtr mesh = MeshManager::getSingleton().createManual(basename + "." + ext, RGN_DEFAULT);
+        mesh->getUserObjectBindings().setUserAny("_AssimpLoaderOptions", opts.options);
+        codec->decode(Root::openFileStream(opts.source), mesh.get());
 
         if (!opts.dest.empty())
         {
@@ -204,9 +222,9 @@ int main(int numargs, char** args)
         }
 
         MeshSerializer meshSer;
-        meshSer.exportMesh(mesh.get(), path + basename + ".mesh");
+        meshSer.exportMesh(mesh, path + basename + ".mesh");
 
-        if (skeleton)
+        if (auto skeleton = mesh->getSkeleton())
         {
             SkeletonSerializer binSer;
             binSer.exportSkeleton(skeleton.get(), path + skeleton->getName());
@@ -219,6 +237,13 @@ int main(int numargs, char** args)
 
         // queue up the materials for serialise
         MaterialSerializer ms;
+#ifdef OGRE_BUILD_COMPONENT_RTSHADERSYSTEM
+        auto& shadergen = RTShader::ShaderGenerator::getSingleton();
+        shadergen.setTargetLanguage("glsl"); // must be valid, but otherwise arbitrary
+        shadergen.getRenderState(MSN_SHADERGEN)->setLightCountAutoUpdate(false);
+        shadergen.validateScheme(MSN_SHADERGEN);
+        ms.addListener(shadergen.getMaterialSerializerListener());
+#endif
         for (const String& name : exportNames)
             ms.queueForExport(MaterialManager::getSingleton().getByName(name));
 
